-----------------------------------------------------------------------------------------------
-- Client Lua Script for SimpleCalculator
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SimpleCalculator Module Definition
-----------------------------------------------------------------------------------------------
local SimpleCalculator = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SimpleCalculator:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SimpleCalculator:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SimpleCalculator OnLoad
-----------------------------------------------------------------------------------------------
function SimpleCalculator:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SimpleCalculator.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SimpleCalculator OnDocLoaded
-----------------------------------------------------------------------------------------------
function SimpleCalculator:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SimpleCalculatorForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("cal", "OnSimpleCalculatorOn", self)


		-- Do additional Addon initialization here
		self.txtTotalItems = self.wndMain:FindChild("txtTotalItems")
		self.txtCost = self.wndMain:FindChild("txtCost")
		
		--CRB_CurrencySprites:sprCashPlatinum	
		self.txtPlat = self.wndMain:FindChild("txtPlat")			
		--CRB_CurrencySprites:sprCashGold
		self.txtGold = self.wndMain:FindChild("txtGold")
		--CRB_CurrencySprites:sprCashSilver
		self.txtSilver = self.wndMain:FindChild("txtSilver")
		--CRB_CurrencySprites:sprCashCopper
		self.txtCopper = self.wndMain:FindChild("txtCopper")

	end
end

-----------------------------------------------------------------------------------------------
-- SimpleCalculator Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
--try catch block for catching non digits
function SimpleCalculator:try(f, catch_f)
	local status, exception = pcall(f)
	if not status then
		catch_f(exception)
	end
end

--for displaying to the screen
function SimpleCalculator:PrintThis(str)
	--Display to system chat
	str = str or "Simple Calculator\n\nUsage: <value> <operator> <value>\n\nUsage example: /cal 2 + 2"
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, str)
end

--turn the string into a table that can be use for calcuations
function SimpleCalculator:FormatArgs(sArgs)
	
	--strip whitespace
	sArgs = string.gsub(sArgs, "%s", "")
	--get operator's position
	local operator_pos = string.find(sArgs, "[+-/*\^]")
	
	if operator_pos then
	--split string into table
		tArgs = {
			string.sub(sArgs, 0, operator_pos-1),
			string.sub(sArgs,operator_pos, operator_pos), 
			string.sub(sArgs,operator_pos+1)
			}
		
		return tArgs
	else
	--return false because operator was missing
		return false
	end
	
end

--return the calculated items of the table
function SimpleCalculator:DoMath(tArgs)

	if tArgs[2] == "+" then
		--add digits
		x = tonumber(tArgs[1])
		y = tonumber(tArgs[3])
		return x + y
		
	elseif tArgs[2] == "-" then
		--subtract digits
		x = tonumber(tArgs[1])
		y = tonumber(tArgs[3])
		return x - y
		
	elseif tArgs[2] == "/" then
		--divide digits
		x = tonumber(tArgs[1])
		y = tonumber(tArgs[3])
		return x / y
		
	elseif tArgs[2] == "*" then
		--multiply digits
		x = tonumber(tArgs[1])
		y = tonumber(tArgs[3])
		return x * y
		
	elseif tArgs[2] == "^" then
		--exponent digits
		x = tonumber(tArgs[1])
		y = tonumber(tArgs[3])
		return x ^ y
		
	else
		return false
	end

end

--reset coins to 00
function SimpleCalculator:ResetCoins()
	self.txtPlat:SetText("00")
	self.txtGold:SetText("00")
	self.txtSilver:SetText("00")
	self.txtCopper:SetText("00")
end


--fill out the coins
function SimpleCalculator:PrintCoins(result)
	result = tostring(result)
	self:ResetCoins()
	--result is a string of numbers 10123100 which needs to be converted to 10P 12G 31S 00C
	
	-- if >8 then max out
	if string.len(result) > 6 then 
		self.txtPlat:SetText("99")
		self.txtGold:SetText("99")
		self.txtSilver:SetText("99")
		self.txtCopper:SetText("99")
		
	else
		if string.len(result) == 1 or string.len(result) == 3 or string.len(result) == 5 or string.len(result) == 7 then
			result = "0" .. result
		end	
		
		--separate the numbers in pairs
		local loopitems = {}
		for num in string.gmatch(result,"%d%d") do
     		loopitems[#loopitems + 1] = tostring(num)
		end
		
		--put them into the correct slots
		if #loopitems == 4 then
			self.txtPlat:SetText(loopitems[1])
			self.txtGold:SetText(loopitems[2])
			self.txtSilver:SetText(loopitems[3])
			self.txtCopper:SetText(loopitems[4])
			
		elseif #loopitems == 3 then
			self.txtPlat:SetText(loopitems[1])
			self.txtGold:SetText(loopitems[2])
			self.txtSilver:SetText(loopitems[3])

		elseif #loopitems == 2 then
			self.txtGold:SetText(loopitems[1])
			self.txtSilver:SetText(loopitems[2])
			
		elseif #loopitems == 1 then
			self.txtSilver:SetText(loopitems[1])
		
		end
	end
		
end


-- on SlashCommand "/cal"
function SimpleCalculator:OnSimpleCalculatorOn(sCommand, sArg)
	self.wndMain:Invoke() -- show the window
	
	--first get the table with the inputed argument
	local tArgs = self:FormatArgs(sArg)
	self:ResetCoins() --set all coins to 00
	
	--if the table isn't false proceed
	if tArgs then
		self:try(function()
				--calculate the items 
				result = self:DoMath(tArgs)
					--if open panel
					if tArgs[2] == "*" then
						self.txtTotalItems:SetText(tArgs[1])
						self.txtCost:SetText(tArgs[3])
						self:PrintCoins(result)					
					end
				--print to system
				self:PrintThis(tArgs[1] ..  tArgs[2] .. tArgs[3] .. " = " .. result)
		end, function(e)
			--error(e)
			self:PrintThis(nil) --Print default usage message
		 end)
	end
	
end


-----------------------------------------------------------------------------------------------
-- SimpleCalculatorForm Functions
-----------------------------------------------------------------------------------------------

-- when the Cancel button is clicked
function SimpleCalculator:OnCancel()
	self.wndMain:Close() -- hide the window
end

-- when the calculate button is clicked
function SimpleCalculator:OnCalculate()
	self:ResetCoins()
	self:try(function()
				tArgs = {self.txtTotalItems:GetText(), "*", self.txtCost:GetText()}
				result = self:DoMath(tArgs)
				self:PrintCoins(result)
				self:PrintThis(tArgs[1] ..  tArgs[2] .. tArgs[3] .. " = " .. result)
		
		end, function(e)
			--error(e)
			self:PrintThis(nil) --Print default usage message
		 end)

end

-----------------------------------------------------------------------------------------------
-- SimpleCalculator Instance
-----------------------------------------------------------------------------------------------
local SimpleCalculatorInst = SimpleCalculator:new()
SimpleCalculatorInst:Init()